import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.awt.datatransfer.*; 

public class EdText extends Frame 
 implements ActionListener {
  TextArea texte = new TextArea();    // La zone de texte
  File fichier=null;                  // Le fichier
  Clipboard pressePapiers;            // Presse-Papiers
  boolean modified=false;             // Indicateur de modifications
  String titre="Editeur de textes ";  // Titre
  ChercheDial chdial;                 // Dialogue recherche
  /**
   * Constructeur de la classe EdText.
   * Dfinition du menu
   */ 
  EdText() {
   // Titre de la fentre et zone de texte
   super("");
   affTitre();
   chdial=new ChercheDial(this);
   add("Center", texte);
   //couteurs d'vnements
   addWindowListener(new Fermeture());
   texte.addTextListener(new ModEvent());
   // Dfinition et affichage du systme de menus
   MenuBar mb=new MenuBar();
   Menu m1=new Menu("Fichier");
   Menu m2=new Menu("Edition");
   Menu m3=new Menu("?");
   mb.add(m1);
   mb.add(m2);
   mb.add(m3);
   ajouteMenuItem("Nouveau","nouveau",m1);
   ajouteMenuItem("Ouvrir...","ouvrir",m1);
   m1.add(new MenuItem("-"));
   ajouteMenuItem("Enregistrer sous...","enregistrersous",m1);
   ajouteMenuItem("Enregistrer","enregistrer",m1);
   m1.add(new MenuItem("-"));
   ajouteMenuItem("Quitter","quitter",m1);
   ajouteMenuItem("Copier","copier",m2);
   ajouteMenuItem("Coller","coller",m2);
   m2.add(new MenuItem("-"));
   ajouteMenuItem("Chercher...","chercher",m2);
   ajouteMenuItem("Suivant","suivant",m2);
   ajouteMenuItem("A propos...","apropos",m3);
   setMenuBar(mb);
   // Taille par dfaut de la frame
   setBounds(50,50,300,400);
   // Presse-Papiers
   pressePapiers=getToolkit().getSystemClipboard();
   // ----------
   pack();
  }

 /**
  * Ajout d'une commande  un menu
  */
 void ajouteMenuItem(String txt, String com, Menu m) {
  MenuItem mi=new MenuItem(txt);
  mi.setActionCommand(com);
  m.add(mi);
  mi.addActionListener(this);
 }
 
 void affTitre() {
  if (fichier==null) setTitle(titre+"[]");
  else setTitle(titre+"["+fichier.getName()+"]");
 }
 
 void quitter() {
  if (modified) saveModif();
  System.exit(0);
 }
 
 void ouvrirFichier() {
  if (modified) saveModif();
  FileDialog f=new FileDialog(this, "Ouvrir", FileDialog.LOAD);
  f.show();
  String nom=f.getFile();
  if (nom!=null) {
   String dir=f.getDirectory();
   fichier=new File(dir, nom);
   lireFichier();
  } 
 }
 
 void lireFichier() {
  try {
   FileInputStream file=new FileInputStream(fichier);
   DataInputStream in=new DataInputStream(file);
   byte[] bw=new byte[in.available()];
   in.readFully(bw);
   in.close();
   texte.setText(new String(bw));
   modified=false;
   affTitre();
  } catch (IOException e) {}
 }
 
 void enregistrerFichierSous() {
  FileDialog f=new FileDialog(this, "Enregistrer", FileDialog.SAVE);
  f.show();
  String nom=f.getFile();
  if (nom!=null) {
   String dir=f.getDirectory();
   fichier=new File(dir,nom);
   sauverFichier();
   affTitre();
  } 
 }

 void enregistrerFichier() {
  if (fichier==null) enregistrerFichierSous();
  else sauverFichier();
 }
 
 void sauverFichier() {
  fichier.delete();
  try {
   String s=texte.getText();
   int l=s.length();
   FileOutputStream fos=new FileOutputStream(fichier);
   OutputStreamWriter osw=new OutputStreamWriter(fos);
   osw.write(s.toCharArray(),0,l);
   osw.close();
  } catch (IOException e) {}
 }

 void saveModif() {
  if (MsgBox.affQuest(this,"Sauver le texte modifi ?"))
   enregistrerFichier();
 }
 
 void copier() {
  String selection=texte.getSelectedText();
  StringSelection clipString=new StringSelection(selection); 
  pressePapiers.setContents(clipString, clipString);
 }
  
 void coller() {
  try {
   StringSelection clipString;
   clipString=(StringSelection)pressePapiers.getContents(this);
   if (clipString!=null) {
    String str=(String)clipString.getTransferData(DataFlavor.stringFlavor);
    texte.insert(str,texte.getCaretPosition());
   } 
  } catch (Exception e) {}
 }

 void chercher(boolean nouveau) {
  if (nouveau) chdial.setVisible(true);
  String mot=chdial.getMot();
  if (mot.length()>0) {
   int pos=texte.getText().indexOf(mot,texte.getCaretPosition());
   if (pos<0) MsgBox.affMsg(this,"Mot absent plus bas dans le texte.");
   else {
    texte.setSelectionStart(pos);     
    texte.setSelectionEnd(pos+mot.length()); 
    //texte.setCaretPosition(pos+mot.length()); 
   }
  }
 }
 
 /**
  * Gestion des vnements lis au systme de menus
  */
 public void actionPerformed(ActionEvent evt) {
  if (evt.getActionCommand().equals("nouveau")) {
   if (modified) saveModif();
   texte.setText("");
   modified=false;
   fichier=null;
   affTitre();
  }
  else if (evt.getActionCommand().equals("ouvrir")) {
   //Fichier/Ouvrir
   ouvrirFichier();
  }
  else if (evt.getActionCommand().equals("enregistrer")) {
   //Fichier/Enregistrer
   enregistrerFichier();
  }
  else if (evt.getActionCommand().equals("enregistrersous")) {
   //Fichier/Enregistrer sous
   enregistrerFichierSous();
  }
  else if (evt.getActionCommand().equals("quitter")) {
  //Fichier/Quitter
   quitter();
  }
  else if (evt.getActionCommand().equals("copier")) {
  //Edition/Copier
   copier();
  }
  else if (evt.getActionCommand().equals("coller")) {
  //Edition/Coller
   coller(); 
  }
  else if (evt.getActionCommand().equals("chercher")) {
  //Edition/Chercher
   chercher(true);
  }
  else if (evt.getActionCommand().equals("suivant")) {
  //Edition/Suivant
   chercher(false);
  } 
  else if (evt.getActionCommand().equals("apropos")) {
  //?/A propos
   MsgBox.affMsg(this,"Un simple diteur de textes.");
  }
 }

 /**
  * Dfinition d'une frame pour l'application et
  * affichage de la fentre
  */
 public static void main(String args[]) {
  System.out.println("Ouverture de l'application...");
  EdText e=new EdText();
  if (args.length>0) {
   e.fichier=new File(args[0]);
   e.lireFichier();
  } 
  e.setVisible(true);
 }
 
 class ModEvent implements TextListener {
  public void textValueChanged(TextEvent e) {
   if (e.getID()==TextEvent.TEXT_VALUE_CHANGED)
    modified=true;
  }
 }
 
 class Fermeture extends WindowAdapter {
  public void windowClosing(WindowEvent event) {
   quitter();
  }
 }

}
